﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;

namespace _2011_04_26_2011_04_27_Praktikum
{
    class FileType
    {
        string extension;
        long numberOfBytes;

        public FileType(string extension, long size)
        {
            this.extension = extension;
            this.numberOfBytes = size;
        }

        public string Extension
        {
            get { return extension; }
        }

        public long NumberOfBytes
        {
            get { return numberOfBytes; }
        }

        public void AddSize(long size)
        {
            numberOfBytes += size;
        }
    }

    class DirectoryStatistics
    {
        List<FileType> fileTypes = new List<FileType>();

        public DirectoryStatistics(string path)
        {
            System.IO.DirectoryInfo di = new System.IO.DirectoryInfo(path);
            System.IO.FileInfo[] files = di.GetFiles();
            for (int i = 0; i < files.Length; i++)
			{
                long size = files[i].Length;
                string ext = files[i].Extension;

                // Dies ginge weniger elegant auch "zu Fuß" mit einer Schleife:
                FileType ft = fileTypes.Find(x => x.Extension == ext);

                if (ft != null)
                {
                    ft.AddSize(size);
                }
                else
                {
                    fileTypes.Add(new FileType(ext, size));
                }
			}

            // Dies ginge weniger elegant auch "zu Fuß" mit Bubble Sort o.ä.:
            // Minus vor CompareTo, damit die großen Einträge nach vorne kommen.
            fileTypes.Sort((x, y) => -x.NumberOfBytes.CompareTo(y.NumberOfBytes));
        }

        const double distanceFromBottom = 20.0;
        const double distanceToNeighbor = 2.0;

        // Am Dateianfang für weniger Schreibarbeit hinzugefügt:
        // using System.Windows.Controls
        public void DrawIn(Canvas canvas)
        {
            if (fileTypes.Count == 0)
            {
                return;
            }

            // Dies ginge weniger elegant auch "zu Fuß" mit einer Schleife:
            long maxNumberOfBytes = fileTypes.Max(x => x.NumberOfBytes);

            if (maxNumberOfBytes == 0)
            {
                return;
            }

            canvas.Children.Clear();
            for (int i = 0; i < fileTypes.Count; i++)
            {
                System.Windows.Shapes.Rectangle rectangle = new System.Windows.Shapes.Rectangle();
                rectangle.Fill = System.Windows.Media.Brushes.LightBlue;
                rectangle.Width = 0.8 * canvas.Width / fileTypes.Count - distanceToNeighbor;
                double x = i * canvas.Width / fileTypes.Count;
                Canvas.SetLeft(rectangle, x);
                double y = (canvas.Height - distanceFromBottom) * fileTypes[i].NumberOfBytes / maxNumberOfBytes;
                y = Math.Max(1.0, y); // damit mindestens ein Pixel dick                
                rectangle.Height = y;
                Canvas.SetBottom(rectangle, distanceFromBottom);
                canvas.Children.Add(rectangle);

                Label label = new Label();
                label.Content = fileTypes[i].Extension;
                Canvas.SetLeft(label, x);
                Canvas.SetTop(label, canvas.Height - distanceFromBottom);
                canvas.Children.Add(label);
            }
        }
    }
}
